#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jul 19 16:48:59 2019

@author: farid
"""
import numpy as np
import os, sys
import zipfile
import argparse


# parse arguments
parser = argparse.ArgumentParser(
  prog='\033[1m\033[94m' + 'python3.6' + '\033[0m' + ' ' + sys.argv[0],
  description='Run CO2Benchmark model and extract outputs.'
)
parser.add_argument('-f', '--file', nargs='+', required=True, help="interface file as input")
parser.add_argument('-of', '--outFile', default='PressureOutput.csv', help="Basename of the written csv file")
parser.add_argument('-p', '--pointidx', type=int, nargs=1, default=441, help='Index of the monitoring point (in 3D)')
parser.add_argument('-t', '--timemax', type=float, nargs=1, default=8640000, help='Index of the monitoring point (in 3D)')
parser.add_argument('-o', '--nobs', type=int, default=10, help='Number of observations in time.')
parser.add_argument('-v', '--verbosity', type=int, default=0, help='Verbosity of the output. 0 = print nothing. 1 = print progress')
args = vars(parser.parse_args())



# Specification
interface = args['file'][0]
outFile = args['outFile']
GridIndex = args['pointidx']
TimeMax=args['timemax']
nObs = args['nobs']
verbose = False if args['verbosity']==0 else True 

# Rename to interface.xml
try:
    os.rename(interface, "interface.xml")
except: 
    pass

# Run the model
NewCommand = 'test_2p ../grids/leakingwellcoarse.dgf %s %s'%(TimeMax, TimeMax/86400)

Process = os.system('timeout 150 ./../%s > log.out 2>&1' %NewCommand)

if verbose:
    print('\nMessage 1:')
    print('\tIf value of \'%d\' is a non-zero value, then compilation problems \n' % Process)


# Read time step from *.pvd
timeVector=[]
with open('benchmark11.pvd') as infile:
    for line in infile:
        if 'timestep=' in line:
            timeVector.append(float((line).split('timestep="')[1].split('" file=')[0]))

converged = TimeMax in timeVector

if converged:
    # Read outputs from vtu files
    Filtepath = sorted([path for path in os.listdir('.') if 'vtu' in path])
    
    output = np.zeros((len(Filtepath)))        
    
    Start = '<DataArray type="Float32" Name="pN" NumberOfComponents="1" format="ascii">'
    End = '</DataArray>'
    
    for idx, filepath in enumerate(Filtepath):
        out=[]
        with open(filepath) as infile:
            copy = False
            for line in infile:
                if line.strip() == Start:
                    copy = True
                    continue
                elif line.strip() == End:
                    copy = False
                    continue
                elif copy:
                    out.append([float(i)/1e5 for i in line.split(' ')[:-1]])
        output[idx] = [val for sublist in out for val in sublist][GridIndex]

    
    # Interpolation
    TimeVector=np.linspace(0, TimeMax, nObs)
    
    interpPressure = np.interp(TimeVector, timeVector, output)

else:
    TimeVector=np.linspace(0, TimeMax, nObs)
    # if not converged give nan
    interpPressure = np.repeat(np.nan, len(TimeVector))  

# Save to csv file
np.savetxt(outFile, np.vstack((TimeVector/86400,interpPressure)).T, fmt='%.8f', delimiter=',',header='Time [day],Pressure [bar]', comments='')

# Zip auxillary files
keys = ['drs', 'dat', 'vtu', 'pvd', 'log']
filePaths = [path for path in os.listdir('.') for key in keys if key in path]

zip_file = zipfile.ZipFile('auxFiles.zip', 'w')
with zip_file:
    # writing each file one by one
    for file in filePaths:
      zip_file.write(file)
      
for path in filePaths: os.remove(path)

if verbose: print('\nauxFiles.zip file has been created successfully!\n')
